@@ -6,7 +6,7 @@
  *  Damien Sauveron <damien.sauveron@labri.fr>
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: readerfactory.c 3200 2008-11-10 12:39:05Z rousseau $
+ * $Id: readerfactory.c 4297 2009-07-03 12:55:47Z rousseau $
  */
 
 /**
@@ -37,6 +37,7 @@
 #include "hotplug.h"
 #include "strlcpycat.h"
 #include "configfile.h"
+#include "utils.h"
 
 #ifndef TRUE
 #define TRUE 1
@@ -47,16 +48,15 @@ static PREADER_CONTEXT sReadersContexts[PCSCLITE_MAX_READERS_CONTEXTS];
 static DWORD dwNumReadersContexts = 0;
 static char *ConfigFile = NULL;
 static int ConfigFileCRC = 0;
+static PCSCLITE_MUTEX LockMutex = PTHREAD_MUTEX_INITIALIZER;
 
 #define IDENTITY_SHIFT 16
 
 LONG RFAllocateReaderSpace(void)
 {
-	int i;   					/* Counter */
+	int i;	/* Counter */
 
-	/*
-	 * Allocate each reader structure
-	 */
+	/* Allocate each reader structure */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
 		sReadersContexts[i] = malloc(sizeof(READER_CONTEXT));
@@ -64,9 +64,7 @@ LONG RFAllocateReaderSpace(void)
 		(sReadersContexts[i])->readerState = NULL;
 	}
 
-	/*
-	 * Create public event structures
-	 */
+	/* Create public event structures */
 	return EHInitializeEventStructures();
 }
 
@@ -104,9 +102,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		return SCARD_E_INVALID_VALUE;
 	}
 
-	/*
-	 * Same name, same port - duplicate reader cannot be used
-	 */
+	/* Same name, same port - duplicate reader cannot be used */
 	if (dwNumReadersContexts != 0)
 	{
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
@@ -132,9 +128,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		}
 	}
 
-	/*
-	 * We must find an empty slot to put the reader structure
-	 */
+	/* We must find an empty slot to put the reader structure */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
 		if ((sReadersContexts[i])->vHandle == 0)
@@ -146,23 +140,19 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 
 	if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 	{
-		/*
-		 * No more spots left return
-		 */
+		/* No more spots left return */
 		return SCARD_E_NO_MEMORY;
 	}
 
-	/*
-	 * Check and set the readername to see if it must be enumerated
-	 */
+	/* Check and set the readername to see if it must be enumerated */
 	parentNode = RFSetReaderName(sReadersContexts[dwContext], lpcReader,
 		lpcLibrary, dwPort, 0);
 	if (parentNode < -1)
 		return SCARD_E_NO_MEMORY;
 
-	strlcpy((sReadersContexts[dwContext])->lpcLibrary, lpcLibrary,
+	(void)strlcpy((sReadersContexts[dwContext])->lpcLibrary, lpcLibrary,
 		sizeof((sReadersContexts[dwContext])->lpcLibrary));
-	strlcpy((sReadersContexts[dwContext])->lpcDevice, lpcDevice,
+	(void)strlcpy((sReadersContexts[dwContext])->lpcDevice, lpcDevice,
 		sizeof((sReadersContexts[dwContext])->lpcDevice));
 	(sReadersContexts[dwContext])->dwVersion = 0;
 	(sReadersContexts[dwContext])->dwPort = dwPort;
@@ -182,9 +172,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	for (i = 0; i < PCSCLITE_MAX_READER_CONTEXT_CHANNELS; i++)
 		(sReadersContexts[dwContext])->psHandles[i].hCard = 0;
 
-	/*
-	 * If a clone to this reader exists take some values from that clone
-	 */
+	/* If a clone to this reader exists take some values from that clone */
 	if (parentNode >= 0 && parentNode < PCSCLITE_MAX_READERS_CONTEXTS)
 	{
 		(sReadersContexts[dwContext])->pdwFeeds =
@@ -197,12 +185,10 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		(sReadersContexts[dwContext])->pdwMutex =
 		  (sReadersContexts[parentNode])->pdwMutex;
 
-		/*
-		 * Call on the driver to see if it is thread safe
-		 */
+		/* Call on the driver to see if it is thread safe */
 		dwGetSize = sizeof(ucThread);
 		rv = IFDGetCapabilities((sReadersContexts[parentNode]),
-		       TAG_IFD_THREAD_SAFE, &dwGetSize, ucThread);
+			TAG_IFD_THREAD_SAFE, &dwGetSize, ucThread);
 
 		if (rv == IFD_SUCCESS && dwGetSize == 1 && ucThread[0] == 1)
 		{
@@ -222,7 +208,6 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		   cloned readers will cause pcscd to crash when
 		   RFUnloadReader unloads the driver library
 		   and there are still devices attached using it --mikeg*/
-
 		*(sReadersContexts[dwContext])->pdwFeeds = 1;
 	}
 
@@ -230,13 +215,12 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	{
 		(sReadersContexts[dwContext])->mMutex =
 			malloc(sizeof(PCSCLITE_MUTEX));
-		SYS_MutexInit((sReadersContexts[dwContext])->mMutex);
+		(void)SYS_MutexInit((sReadersContexts[dwContext])->mMutex);
 	}
 
 	if ((sReadersContexts[dwContext])->pdwMutex == NULL)
 	{
 		(sReadersContexts[dwContext])->pdwMutex = malloc(sizeof(DWORD));
-
 		*(sReadersContexts[dwContext])->pdwMutex = 1;
 	}
 
@@ -247,13 +231,11 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	{
 		/* Cannot connect to reader. Exit gracefully */
 		Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-		RFRemoveReader(lpcReader, dwPort);
+		(void)RFRemoveReader(lpcReader, dwPort);
 		return rv;
 	}
 
-	/*
-	 * asynchronous card movement?
-	 */
+	/* asynchronous card movement?  */
 	{
 		RESPONSECODE (*fct)(DWORD) = NULL;
 
@@ -273,30 +255,23 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (rv != SCARD_S_SUCCESS)
 		{
 			Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-			RFRemoveReader(lpcReader, dwPort);
+			(void)RFRemoveReader(lpcReader, dwPort);
 			return rv;
 		}
 	}
 
-	/*
-	 * Call on the driver to see if there are multiple slots
-	 */
-
+	/* Call on the driver to see if there are multiple slots */
 	dwGetSize = sizeof(ucGetData);
 	rv = IFDGetCapabilities((sReadersContexts[dwContext]),
 		TAG_IFD_SLOTS_NUMBER, &dwGetSize, ucGetData);
 
 	if (rv != IFD_SUCCESS || dwGetSize != 1 || ucGetData[0] == 0)
-		/*
-		 * Reader does not have this defined.  Must be a single slot
-		 * reader so we can just return SCARD_S_SUCCESS.
-		 */
+		/* Reader does not have this defined.  Must be a single slot
+		 * reader so we can just return SCARD_S_SUCCESS. */
 		return SCARD_S_SUCCESS;
 
 	if (rv == IFD_SUCCESS && dwGetSize == 1 && ucGetData[0] == 1)
-		/*
-		 * Reader has this defined and it only has one slot
-		 */
+		/* Reader has this defined and it only has one slot */
 		return SCARD_S_SUCCESS;
 
 	/*
@@ -304,20 +279,14 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 	 * structure for each one accordingly
 	 */
 
-	/*
-	 * Initialize the rest of the slots
-	 */
-
+	/* Initialize the rest of the slots */
 	for (j = 1; j < ucGetData[0]; j++)
 	{
 		char *tmpReader = NULL;
 		DWORD dwContextB = 0;
 		RESPONSECODE (*fct)(DWORD) = NULL;
 
-		/*
-		 * We must find an empty spot to put the
-		 * reader structure
-		 */
+		/* We must find an empty spot to put the reader structure */
 		for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		{
 			if ((sReadersContexts[i])->vHandle == 0)
@@ -329,24 +298,20 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 
 		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 		{
-			/*
-			 * No more slot left return
-			 */
+			/* No more slot left return */
 			rv = RFRemoveReader(lpcReader, dwPort);
 			return SCARD_E_NO_MEMORY;
 		}
 
-		/*
-		 * Copy the previous reader name and increment the slot number
-		 */
+		/* Copy the previous reader name and increment the slot number */
 		tmpReader = sReadersContexts[dwContextB]->lpcReader;
-		strlcpy(tmpReader, sReadersContexts[dwContext]->lpcReader,
+		(void)strlcpy(tmpReader, sReadersContexts[dwContext]->lpcReader,
 			sizeof(sReadersContexts[dwContextB]->lpcReader));
 		sprintf(tmpReader + strlen(tmpReader) - 2, "%02X", j);
 
-		strlcpy((sReadersContexts[dwContextB])->lpcLibrary, lpcLibrary,
+		(void)strlcpy((sReadersContexts[dwContextB])->lpcLibrary, lpcLibrary,
 			sizeof((sReadersContexts[dwContextB])->lpcLibrary));
-		strlcpy((sReadersContexts[dwContextB])->lpcDevice, lpcDevice,
+		(void)strlcpy((sReadersContexts[dwContextB])->lpcDevice, lpcDevice,
 			sizeof((sReadersContexts[dwContextB])->lpcDevice));
 		(sReadersContexts[dwContextB])->dwVersion =
 		  (sReadersContexts[dwContext])->dwVersion;
@@ -355,9 +320,9 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		(sReadersContexts[dwContextB])->vHandle =
 		  (sReadersContexts[dwContext])->vHandle;
 		(sReadersContexts[dwContextB])->mMutex =
-		   (sReadersContexts[dwContext])->mMutex;
+		  (sReadersContexts[dwContext])->mMutex;
 		(sReadersContexts[dwContextB])->pdwMutex =
-		   (sReadersContexts[dwContext])->pdwMutex;
+		  (sReadersContexts[dwContext])->pdwMutex;
 		sReadersContexts[dwContextB]->dwSlot =
 			sReadersContexts[dwContext]->dwSlot + j;
 
@@ -365,7 +330,6 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		 * Added by Dave - slots did not have a pdwFeeds
 		 * parameter so it was by luck they were working
 		 */
-
 		(sReadersContexts[dwContextB])->pdwFeeds =
 		  (sReadersContexts[dwContext])->pdwFeeds;
 
@@ -383,10 +347,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		for (i = 0; i < PCSCLITE_MAX_READER_CONTEXT_CHANNELS; i++)
 			(sReadersContexts[dwContextB])->psHandles[i].hCard = 0;
 
-		/*
-		 * Call on the driver to see if the slots are thread safe
-		 */
-
+		/* Call on the driver to see if the slots are thread safe */
 		dwGetSize = sizeof(ucThread);
 		rv = IFDGetCapabilities((sReadersContexts[dwContext]),
 			TAG_IFD_SLOT_THREAD_SAFE, &dwGetSize, ucThread);
@@ -395,7 +356,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		{
 			(sReadersContexts[dwContextB])->mMutex =
 				malloc(sizeof(PCSCLITE_MUTEX));
-			SYS_MutexInit((sReadersContexts[dwContextB])->mMutex);
+			(void)SYS_MutexInit((sReadersContexts[dwContextB])->mMutex);
 
 			(sReadersContexts[dwContextB])->pdwMutex = malloc(sizeof(DWORD));
 			*(sReadersContexts[dwContextB])->pdwMutex = 1;
@@ -409,7 +370,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (rv != SCARD_S_SUCCESS)
 		{
 			/* Cannot connect to slot. Exit gracefully */
-			RFRemoveReader(lpcReader, dwPort);
+			(void)RFRemoveReader(lpcReader, dwPort);
 			return rv;
 		}
 
@@ -430,7 +391,7 @@ LONG RFAddReader(LPSTR lpcReader, DWORD dwPort, LPSTR lpcLibrary, LPSTR lpcDevic
 		if (rv != SCARD_S_SUCCESS)
 		{
 			Log2(PCSC_LOG_ERROR, "%s init failed.", lpcReader);
-			RFRemoveReader(lpcReader, dwPort);
+			(void)RFRemoveReader(lpcReader, dwPort);
 			return rv;
 		}
 	}
@@ -451,18 +412,14 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 	{
 		int i;
 
-		/*
-		 * Try to destroy the thread
-		 */
+		/* Try to destroy the thread */
 		rv = EHDestroyEventHandler(sContext);
 
 		rv = RFUnInitializeReader(sContext);
 		if (rv != SCARD_S_SUCCESS)
 			return rv;
 
-		/*
-		 * Destroy and free the mutex
-		 */
+		/* Destroy and free the mutex */
 		if ((NULL == sContext->pdwMutex) || (NULL == sContext->pdwFeeds))
 		{
 			Log1(PCSC_LOG_ERROR,
@@ -472,7 +429,7 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 
 		if (*sContext->pdwMutex == 1)
 		{
-			SYS_MutexDestroy(sContext->mMutex);
+			(void)SYS_MutexDestroy(sContext->mMutex);
 			free(sContext->mMutex);
 		}
 
@@ -511,6 +468,9 @@ LONG RFRemoveReader(LPSTR lpcReader, DWORD dwPort)
 			sContext->psHandles[i].hCard = 0;
 
 		dwNumReadersContexts -= 1;
+
+		/* signal an event to clients */
+		StatSynchronize(NULL);
 	}
 
 	return SCARD_S_SUCCESS;
@@ -526,9 +486,7 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 	int usedDigits[PCSCLITE_MAX_READERS_CONTEXTS];
 	int i;
 
-	/*
-	 * Clear the list
-	 */
+	/* Clear the list */
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 		usedDigits[i] = FALSE;
 
@@ -543,9 +501,7 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 					UCHAR tagValue[1];
 					LONG ret;
 
-					/*
-					 * Ask the driver if it supports multiple channels
-					 */
+					/* Ask the driver if it supports multiple channels */
 					valueLength = sizeof(tagValue);
 					ret = IFDGetCapabilities((sReadersContexts[i]),
 						TAG_IFD_SIMULTANEOUS_ACCESS,
@@ -561,10 +517,7 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 					else
 						supportedChannels = 1;
 
-					/*
-					 * Check to see if it is a hotplug reader and
-					 * different
-					 */
+					/* Check to see if it is a hotplug reader and different */
 					if (((((sReadersContexts[i])->dwPort & 0xFFFF0000) ==
 							PCSCLITE_HP_BASE_PORT)
 						&& ((sReadersContexts[i])->dwPort != dwPort))
@@ -586,15 +539,12 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 						 */
 						currentDigit = strtol(lpcReader + strlen(lpcReader) - 5, NULL, 16);
 
-						/*
-						 * This spot is taken
-						 */
+						/* This spot is taken */
 						usedDigits[currentDigit] = TRUE;
 					}
 				}
 			}
 		}
-
 	}
 
 	/* default value */
@@ -625,11 +575,10 @@ LONG RFSetReaderName(PREADER_CONTEXT rContext, LPSTR readerName,
 		}
 	}
 
-	sprintf(rContext->lpcReader, "%s %02X %02lX", readerName, i, dwSlot);
+	snprintf(rContext->lpcReader, sizeof(rContext->lpcReader), "%s %02X %02lX",
+		readerName, i, dwSlot);
 
-	/*
-	 * Set the slot in 0xDDDDCCCC
-	 */
+	/* Set the slot in 0xDDDDCCCC */
 	rContext->dwSlot = (i << 16) + dwSlot;
 
 	return parent;
@@ -768,9 +717,7 @@ LONG RFReaderInfoById(DWORD dwIdentity, PREADER_CONTEXT * sReader)
 {
 	int i;
 
-	/*
-	 * Strip off the lower nibble and get the identity
-	 */
+	/* Strip off the lower nibble and get the identity */
 	dwIdentity = dwIdentity >> IDENTITY_SHIFT;
 	dwIdentity = dwIdentity << IDENTITY_SHIFT;
 
@@ -792,9 +739,7 @@ LONG RFLoadReader(PREADER_CONTEXT rContext)
 	{
 		Log2(PCSC_LOG_INFO, "Reusing already loaded driver for %s",
 			rContext->lpcLibrary);
-		/*
-		 * Another reader exists with this library loaded
-		 */
+		/* Another reader exists with this library loaded */
 		return SCARD_S_SUCCESS;
 	}
 
@@ -822,37 +767,26 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 
 	if (rv1 != SCARD_S_SUCCESS && rv2 != SCARD_S_SUCCESS && rv3 != SCARD_S_SUCCESS)
 	{
-		/*
-		 * Neither version of the IFD Handler was found - exit
-		 */
+		/* Neither version of the IFD Handler was found - exit */
 		Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing");
 
 		exit(1);
 	} else if (rv1 == SCARD_S_SUCCESS)
 	{
-		/*
-		 * Ifd Handler 1.0 found
-		 */
+		/* Ifd Handler 1.0 found */
 		rContext->dwVersion = IFD_HVERSION_1_0;
 	} else if (rv3 == SCARD_S_SUCCESS)
 	{
-		/*
-		 * Ifd Handler 3.0 found
-		 */
+		/* Ifd Handler 3.0 found */
 		rContext->dwVersion = IFD_HVERSION_3_0;
 	}
 	else
 	{
-		/*
-		 * Ifd Handler 2.0 found
-		 */
+		/* Ifd Handler 2.0 found */
 		rContext->dwVersion = IFD_HVERSION_2_0;
 	}
 
-	/*
-	 * The following binds version 1.0 of the IFD Handler specs
-	 */
-
+	/* The following binds version 1.0 of the IFD Handler specs */
 	if (rContext->dwVersion == IFD_HVERSION_1_0)
 	{
 		Log1(PCSC_LOG_INFO, "Loading IFD Handler 1.0");
@@ -872,7 +806,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 		Log1(PCSC_LOG_CRITICAL, "IFDHandler functions missing: " #function ); \
 		exit(1); )
 
-		DYN_GetAddress(rContext->vHandle, &f, "IO_Create_Channel");
+		(void)DYN_GetAddress(rContext->vHandle, &f, "IO_Create_Channel");
 		rContext->psFunctions.psFunctions_v1.pvfCreateChannel = f;
 
 		if (SCARD_S_SUCCESS != DYN_GetAddress(rContext->vHandle, &f,
@@ -893,10 +827,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	}
 	else if (rContext->dwVersion == IFD_HVERSION_2_0)
 	{
-		/*
-		 * The following binds version 2.0 of the IFD Handler specs
-		 */
-
+		/* The following binds version 2.0 of the IFD Handler specs */
 #define GET_ADDRESS_OPTIONALv2(s, code) \
 { \
 	void *f1 = NULL; \
@@ -927,10 +858,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	}
 	else if (rContext->dwVersion == IFD_HVERSION_3_0)
 	{
-		/*
-		 * The following binds version 3.0 of the IFD Handler specs
-		 */
-
+		/* The following binds version 3.0 of the IFD Handler specs */
 #define GET_ADDRESS_OPTIONALv3(s, code) \
 { \
 	void *f1 = NULL; \
@@ -962,9 +890,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 	}
 	else
 	{
-		/*
-		 * Who knows what could have happenned for it to get here.
-		 */
+		/* Who knows what could have happenned for it to get here. */
 		Log1(PCSC_LOG_CRITICAL, "IFD Handler not 1.0/2.0 or 3.0");
 		exit(1);
 	}
@@ -974,10 +900,7 @@ LONG RFBindFunctions(PREADER_CONTEXT rContext)
 
 LONG RFUnBindFunctions(PREADER_CONTEXT rContext)
 {
-	/*
-	 * Zero out everything
-	 */
-
+	/* Zero out everything */
 	memset(&rContext->psFunctions, 0, sizeof(rContext->psFunctions));
 
 	return SCARD_S_SUCCESS;
@@ -985,14 +908,11 @@ LONG RFUnBindFunctions(PREADER_CONTEXT rContext)
 
 LONG RFUnloadReader(PREADER_CONTEXT rContext)
 {
-	/*
-	 * Make sure no one else is using this library
-	 */
-
+	/* Make sure no one else is using this library */
 	if (*rContext->pdwFeeds == 1)
 	{
 		Log1(PCSC_LOG_INFO, "Unloading reader driver.");
-		DYN_CloseLibrary(&rContext->vHandle);
+		(void)DYN_CloseLibrary(&rContext->vHandle);
 	}
 
 	rContext->vHandle = NULL;
@@ -1014,24 +934,25 @@ LONG RFCheckSharing(DWORD hCard)
 		return SCARD_S_SUCCESS;
 	else
 		return SCARD_E_SHARING_VIOLATION;
-
 }
 
 LONG RFLockSharing(DWORD hCard)
 {
 	PREADER_CONTEXT rContext = NULL;
+	LONG rv;
 
-	RFReaderInfoById(hCard, &rContext);
+	(void)RFReaderInfoById(hCard, &rContext);
 
-	if (RFCheckSharing(hCard) == SCARD_S_SUCCESS)
+	(void)SYS_MutexLock(&LockMutex);
+	rv = RFCheckSharing(hCard);
+	if (SCARD_S_SUCCESS == rv)
 	{
 		rContext->LockCount += 1;
 		rContext->dwLockId = hCard;
 	}
-	else
-		return SCARD_E_SHARING_VIOLATION;
+	(void)SYS_MutexUnLock(&LockMutex);
 
-	return SCARD_S_SUCCESS;
+	return rv;
 }
 
 LONG RFUnlockSharing(DWORD hCard)
@@ -1043,16 +964,39 @@ LONG RFUnlockSharing(DWORD hCard)
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
+	(void)SYS_MutexLock(&LockMutex);
 	rv = RFCheckSharing(hCard);
+	if (SCARD_S_SUCCESS == rv)
+	{
+		if (rContext->LockCount > 0)
+			rContext->LockCount -= 1;
+		if (0 == rContext->LockCount)
+			rContext->dwLockId = 0;
+	}
+	(void)SYS_MutexUnLock(&LockMutex);
+
+	return rv;
+}
+
+LONG RFUnlockAllSharing(DWORD hCard)
+{
+	PREADER_CONTEXT rContext = NULL;
+	LONG rv;
+
+	rv = RFReaderInfoById(hCard, &rContext);
 	if (rv != SCARD_S_SUCCESS)
 		return rv;
 
-	if (rContext->LockCount > 0)
-		rContext->LockCount -= 1;
-	if (0 == rContext->LockCount)
+	(void)SYS_MutexLock(&LockMutex);
+	rv = RFCheckSharing(hCard);
+	if (SCARD_S_SUCCESS == rv)
+	{
+		rContext->LockCount = 0;
 		rContext->dwLockId = 0;
+	}
+	(void)SYS_MutexUnLock(&LockMutex);
 
-	return SCARD_S_SUCCESS;
+	return rv;
 }
 
 LONG RFUnblockContext(SCARDCONTEXT hContext)
@@ -1075,17 +1019,11 @@ LONG RFInitializeReader(PREADER_CONTEXT rContext)
 {
 	LONG rv;
 
-	/*
-	 * Spawn the event handler thread
-	 */
+	/* Spawn the event handler thread */
 	Log3(PCSC_LOG_INFO, "Attempting startup of %s using %s",
 		rContext->lpcReader, rContext->lpcLibrary);
 
-  /******************************************/
-	/*
-	 * This section loads the library
-	 */
-  /******************************************/
+	/* loads the library */
 	rv = RFLoadReader(rContext);
 	if (rv != SCARD_S_SUCCESS)
 	{
@@ -1093,35 +1031,29 @@ LONG RFInitializeReader(PREADER_CONTEXT rContext)
 		return rv;
 	}
 
-  /*******************************************/
-	/*
-	 * This section binds the functions
-	 */
-  /*******************************************/
+	/* binds the functions */
 	rv = RFBindFunctions(rContext);
 
 	if (rv != SCARD_S_SUCCESS)
 	{
 		Log2(PCSC_LOG_ERROR, "RFBindFunctions failed: %X", rv);
-		RFUnloadReader(rContext);
+		(void)RFUnloadReader(rContext);
 		return rv;
 	}
 
-  /*******************************************/
-	/*
-	 * This section tries to open the port
-	 */
-  /*******************************************/
-
+	/* tries to open the port */
 	rv = IFDOpenIFD(rContext);
 
 	if (rv != IFD_SUCCESS)
 	{
 		Log3(PCSC_LOG_CRITICAL, "Open Port %X Failed (%s)",
 			rContext->dwPort, rContext->lpcDevice);
-		RFUnBindFunctions(rContext);
-		RFUnloadReader(rContext);
-		return SCARD_E_INVALID_TARGET;
+		(void)RFUnBindFunctions(rContext);
+		(void)RFUnloadReader(rContext);
+		if (IFD_NO_SUCH_DEVICE == rv)
+			return SCARD_E_UNKNOWN_READER;
+		else
+			return SCARD_E_INVALID_TARGET;
 	}
 
 	return SCARD_S_SUCCESS;
@@ -1132,9 +1064,7 @@ LONG RFUnInitializeReader(PREADER_CONTEXT rContext)
 	Log2(PCSC_LOG_INFO, "Attempting shutdown of %s.",
 		rContext->lpcReader);
 
-	/*
-	 * Close the port, unbind the functions, and unload the library
-	 */
+	/* Close the port, unbind the functions, and unload the library */
 
 	/*
 	 * If the reader is getting uninitialized then it is being unplugged
@@ -1142,9 +1072,9 @@ LONG RFUnInitializeReader(PREADER_CONTEXT rContext)
 	 *
 	 * IFDPowerICC( rContext, IFD_POWER_DOWN, Atr, &AtrLen );
 	 */
-	IFDCloseIFD(rContext);
-	RFUnBindFunctions(rContext);
-	RFUnloadReader(rContext);
+	(void)IFDCloseIFD(rContext);
+	(void)RFUnBindFunctions(rContext);
+	(void)RFUnloadReader(rContext);
 
 	return SCARD_S_SUCCESS;
 }
@@ -1153,10 +1083,8 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 {
 	USHORT randHandle;
 
-	/*
-	 * Create a random handle with 16 bits check to see if it already is
-	 * used.
-	 */
+	/* Create a random handle with 16 bits check to see if it already is
+	 * used. */
 	randHandle = SYS_RandomInt(10, 65000);
 
 	while (1)
@@ -1174,9 +1102,7 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 					if ((rContext->dwIdentity + randHandle) ==
 						(sReadersContexts[i])->psHandles[j].hCard)
 					{
-						/*
-						 * Get a new handle and loop again
-						 */
+						/* Get a new handle and loop again */
 						randHandle = SYS_RandomInt(10, 65000);
 						continue;
 					}
@@ -1184,11 +1110,8 @@ SCARDHANDLE RFCreateReaderHandle(PREADER_CONTEXT rContext)
 			}
 		}
 
-		/*
-		 * Once the for loop is completed w/o restart a good handle was
-		 * found and the loop can be exited.
-		 */
-
+		/* Once the for loop is completed w/o restart a good handle was
+		 * found and the loop can be exited. */
 		if (i == PCSCLITE_MAX_READERS_CONTEXTS)
 			break;
 	}
@@ -1217,8 +1140,9 @@ LONG RFFindReaderHandle(SCARDHANDLE hCard)
 	return SCARD_E_INVALID_HANDLE;
 }
 
-LONG RFDestroyReaderHandle(SCARDHANDLE hCard)
+LONG RFDestroyReaderHandle(/*@unused@*/ SCARDHANDLE hCard)
 {
+	(void)hCard;
 	return SCARD_S_SUCCESS;
 }
 
@@ -1268,9 +1192,7 @@ LONG RFSetReaderEventState(PREADER_CONTEXT rContext, DWORD dwEvent)
 {
 	int i;
 
-	/*
-	 * Set all the handles for that reader to the event
-	 */
+	/* Set all the handles for that reader to the event */
 	for (i = 0; i < PCSCLITE_MAX_READER_CONTEXT_CHANNELS; i++)
 	{
 		if (rContext->psHandles[i].hCard != 0)
@@ -1358,9 +1280,7 @@ void RFCleanupReaders(int shouldExit)
 
 			strncpy(lpcStripReader, (sReadersContexts[i])->lpcReader,
 				sizeof(lpcStripReader));
-			/*
-			 * strip the 6 last char ' 00 00'
-			 */
+			/* strip the 6 last char ' 00 00' */
 			lpcStripReader[strlen(lpcStripReader) - 6] = '\0';
 
 			rv = RFRemoveReader(lpcStripReader, sReadersContexts[i]->dwPort);
@@ -1370,10 +1290,7 @@ void RFCleanupReaders(int shouldExit)
 		}
 	}
 
-	/*
-	 * exit() will call at_exit()
-	 */
-
+	/* exit() will call at_exit() */
 	if (shouldExit)
 		exit(0);
 }
@@ -1396,7 +1313,8 @@ int RFStartSerialReaders(const char *readerconf)
 	{
 		int j;
 
-		RFAddReader(reader_list[i].pcFriendlyname, reader_list[i].dwChannelId,
+		(void)RFAddReader(reader_list[i].pcFriendlyname,
+			reader_list[i].dwChannelId,
 			reader_list[i].pcLibpath, reader_list[i].pcDevicename);
 
 		/* update the ConfigFileCRC (this false "CRC" is very weak) */
@@ -1422,7 +1340,7 @@ void RFReCheckReaderConf(void)
 	SerialReader *reader_list;
 	int i, crc;
 
-	DBGetReaderList(ConfigFile, &reader_list);
+	(void)DBGetReaderList(ConfigFile, &reader_list);
 
 	/* the list is empty */
 	if (NULL == reader_list)
@@ -1488,7 +1406,7 @@ void RFReCheckReaderConf(void)
 					{
 						Log2(PCSC_LOG_INFO, "Reader %s disappeared",
 							reader_list[i].pcFriendlyname);
-						RFRemoveReader(reader_list[i].pcFriendlyname,
+						(void)RFRemoveReader(reader_list[i].pcFriendlyname,
 							reader_list[r].dwChannelId);
 					}
 				}
@@ -1498,7 +1416,7 @@ void RFReCheckReaderConf(void)
 		/* the reader was not present */
 		if (!present)
 			/* we try to add it */
-			RFAddReader(reader_list[i].pcFriendlyname,
+			(void)RFAddReader(reader_list[i].pcFriendlyname,
 				reader_list[i].dwChannelId, reader_list[i].pcLibpath,
 				reader_list[i].pcDevicename);
 
@@ -1518,11 +1436,10 @@ void RFSuspendAllReaders(void)
 	{
 		if ((sReadersContexts[i])->vHandle != 0)
 		{
-			EHDestroyEventHandler(sReadersContexts[i]);
-			IFDCloseIFD(sReadersContexts[i]);
+			(void)EHDestroyEventHandler(sReadersContexts[i]);
+			(void)IFDCloseIFD(sReadersContexts[i]);
 		}
 	}
-
 }
 
 void RFAwakeAllReaders(void)
@@ -1536,15 +1453,15 @@ void RFAwakeAllReaders(void)
 	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
 	{
 		/* If the library is loaded and the event handler is not running */
-		if ( ((sReadersContexts[i])->vHandle   != 0) &&
-		     ((sReadersContexts[i])->pthThread == 0) )
+		if ( ((sReadersContexts[i])->vHandle != 0) &&
+			((sReadersContexts[i])->pthThread == 0) )
 		{
 			int j;
 
 			for (j=0; j < i; j++)
 			{
 				if (((sReadersContexts[j])->vHandle == (sReadersContexts[i])->vHandle)&&
-					((sReadersContexts[j])->dwPort   == (sReadersContexts[i])->dwPort))
+					((sReadersContexts[j])->dwPort == (sReadersContexts[i])->dwPort))
 				{
 					initFlag = 1;
 				}
@@ -1561,9 +1478,8 @@ void RFAwakeAllReaders(void)
 					(sReadersContexts[i])->dwPort, (sReadersContexts[i])->lpcDevice);
 			}
 
-
-			EHSpawnEventHandler(sReadersContexts[i], NULL);
-			RFSetReaderEventState(sReadersContexts[i], SCARD_RESET);
+			(void)EHSpawnEventHandler(sReadersContexts[i], NULL);
+			(void)RFSetReaderEventState(sReadersContexts[i], SCARD_RESET);
 		}
 	}
 }
